//******************************************************************************
// Projeto - RELOGIO BINARIO (NERD) - mostra a hora, minutos e segundos
// no formato binario, atravs de um painel (matriz) com 20 LEDs
//
// Desenvolvido por Mrcio Jos Soares (03/01/2006)
//
// Arquivo - relogio_nerd.c (verso 1.0)
//
// Usa a frequencia de 60 Hz, retirada da rede eltrica para "marcar o tempo"
//
// Microcontrolador utilizado: PIC16F628-04/P (no testado com verso "A")
// Compilador: CCS C Compiler - verso PCM 3.15
//
// Pinos utilizados:
// RA0 - controle da linha 1 na matriz de LEDs do mostrador
// RA1 - controle da linha 2 na matriz de LEDs do mostrador
// RA2 - controle da linha 3 na matriz de LEDs do mostrador
// RA3 - controle da linha 4 na matriz de LEDs do mostrador
// RA4 - modo ajuste (avana horario)
// RA5 - modo ajuste (muda de minutos para horas)
// RA6 - no utilizado
// RA7 - no utilizado
//
// RB0 - entrada para clock 60 Hz (interrupo)
// RB1 - jumper para formato 24 horas
// RB2 - controle da coluna 1 na matriz de LEDs do mostrador
// RB3 - controle da coluna 2 na matriz de LEDs do mostrador
// RB4 - controle da coluna 3 na matriz de LEDs do mostrador
// RB5 - controle da coluna 4 na matriz de LEDs do mostrador
// RB6 - controle da coluna 5 na matriz de LEDs do mostrador
// RB7 - controle da coluna 6 na matriz de LEDs do mostrador
//
// Obs:  utilizado ULN2003 para controle das linhas da matriz de LEDs
//******************************************************************************

//******************************************************************************
// Este cdigo-fonte  parte integrante do artigo Relgio Binrio com
// microcontrolador. Maiores informaes sobre o projeto podem ser obtidas
// no referido artigo publicado na Revista Eletronica Total
//******************************************************************************

//******************************************************************************
// O Copyright deste programa est reservado para Mrcio Jos Soares e seu
// contedo est protegido pela Lei de Direitos Autorais LEI N 9.610,
// de 19 de Fevereiro de 1998.  estritamente proibida o uso comercial como
// tambm a publicao total ou parcial dos contedos aqui apresentados
// sem prvia autorizao, por escrito, do autor.
//******************************************************************************

//******************************************************************************
// Arquivos includos neste mdulo
//******************************************************************************
#include "relogio_nerd.h"

//******************************************************************************
// Funo para tratar a interrupo externa em RB0
// Conta 60 interrupes
// recebe 	- nada
// retorna 	- nada
//******************************************************************************
#int_EXT
void EXT_isr(){

   restart_WDT();	                     //zera WDT

   conta++;                            //incrementa contagem

   if (conta == 60){                   //igual a 60?
      conta=0;                         //zera
      ++horario.segundos;              //incrementa segundos

      if (horario.segundos > 59){      //igual a 60?
         horario.segundos = 0;         //retorna ao valor inicial
         ++horario.minutos;            //incrementa minutos

         if (horario.minutos > 59){   //igual a 60?
            horario.minutos = 0;       //retorna ao valor inicial
            ++horario.horas;           //incrementa horas

            if (horario.horas > hora_final){   //igual a 24 ou 12?
               horario.horas = hora_inicial;      //retorna tudo ao valor inicial
               horario.minutos = 0;
               horario.segundos = 0;
            }
         }
      }
   }
   restart_WDT();	                     //zera WDT
}

//******************************************************************************
// Funo para desligar os LEDs (LINHAS e COLUNAS)
// recebe 	- nada
// retorna 	- nada
//******************************************************************************
void zera_lin_col(void){

   output_low(LIN1);                //zera linhas
   output_low(LIN2);
   output_low(LIN3);
   output_low(LIN4);

   output_high(COL1);              //zera colunas
   output_high(COL2);
   output_high(COL3);
   output_high(COL4);
   output_high(COL5);
   output_high(COL6);
   restart_WDT();	                 //zera WDT
}

//******************************************************************************
// Funo para marcar qual ajuste ser realizado
// Acende todos os LEDs da coluna 6, 4 ou 2
// recebe 	- coluna
// retorna 	- nada
//******************************************************************************
void marca_ajuste(unsigned char col){

   LINHAS |= 0x0F;               //todas linhas ligadas
   switch (col){
      case 2:
         output_low(COL2);
         break;
      case 4:
         output_low(COL4);
         break;
      case 6:
         output_low(COL6);
         break;
    }
    delay_ms(1000);              //temporiza
    zera_lin_col();              //zera os pinos de I/O do controle da matriz
    restart_WDT();	            //zera WDT
}

//******************************************************************************
// Funo para piscar todos os LEDs em sequencia
// Acende todos os LEDs da coluna 6 a coluna 1
// recebe 	- nada
// retorna 	- nada
//******************************************************************************
void pisca_ajuste(void){

   unsigned char i, j;

   j=0x80;
   LINHAS = 0x0F;             //insere o valor mximo possvel
   for(i=0;i<6;i++){
      COLUNAS = ~j;           //liga coluna
      delay_ms(500);          //temporiza
      COLUNAS = 0xFF;         //desliga coluna
      j >>= 1;                //desloca a direita
   }

   restart_WDT();	            //zera WDT
}

//******************************************************************************
// Funo para ajustar o horario
// Atualiza a matriz de LEDs a partir de zero hora
// recebe 	- nada
// retorna 	- nada
//******************************************************************************
void ajuste_horario(void){

   switch (est_ajt){                     //verifica estado inicial do ajuste
      case 0:                            //entrada no modo ajuste
         delay_ms(300);
         disable_interrupts(INT_EXT);    //desabilita interrupo no port RB0
         disable_interrupts(GLOBAL);     //desabilita interrupo global
         zera_lin_col();                 //zera os pinos de I/O do controle
                                         //da matriz

         if (prima)
            //pisca todos os LEDs para informar que aceitou
            //entrar no modo ajuste
            pisca_ajuste();
         else
            //se ajuste por falta de energia ou ainda por reset
            //no pisca para informar, e marca para prxima vez
            prima=1;

         horario.segundos=0;              //prepara para ajustar segundos
         horario.minutos=0;
         horario.horas=0;
         marca_ajuste(4);                 //informa que vai ajustar minutos
         est_ajt=1;                       //muda mquina de estado
         break;

      case 1:
         if (!input(AJT1)){
            delay_ms(300);
            ++horario.minutos;
            if (horario.minutos > 59)    //se igual a 60, zera
               horario.minutos = 0;
         }
         if (!input(AJT2)){              //S2 continua ajustes, horas
            delay_ms(300);
            est_ajt=2;
            marca_ajuste(2);             //informa que vai ajustar horas
         }
         break;

      case 2:
         if (!input(AJT1)){
            delay_ms(300);
            ++horario.horas;
            if (horario.horas > hora_final)   //se igual a 24 ou 12, zera
               horario.horas = hora_inicial;
         }
         if (!input(AJT2)){                   //S2 encerra ajustes
            delay_ms(300);
            est_ajt=0;
            conta=0;
            //pisca todos os LEDs para informar que aceitou
            //ajustes feitos
            pisca_ajuste();

            enable_interrupts(INT_EXT);      //desabilita interrupo no port RB0
            enable_interrupts(GLOBAL);       //desabilita interrupo global
         }
         break;

   }
   restart_WDT();	                   //zera WDT

}

//******************************************************************************
// Funo para ligar/desligar matriz de LEDs
// recebe 	- nmero da coluna a ser ligada
// retorna 	- nada
//******************************************************************************
void pisca(unsigned char nr){

   unsigned char tempo = 3;         //tempo para varredura (0,003 segundos)

 	restart_WDT();					      //zera WDT
   switch (nr){
      case 1:
         output_low(COL1);          //liga coluna
         delay_ms(tempo);           //temporiza
         output_high(COL1);         //desliga coluna
         break;
      case 2:
         output_low(COL2);          //liga coluna
         delay_ms(tempo);           //temporiza
         output_high(COL2);         //desliga coluna
         break;
      case 3:
         output_low(COL3);          //liga coluna
         delay_ms(tempo);           //temporiza
         output_high(COL3);         //desliga coluna
         break;
      case 4:
         output_low(COL4);          //liga coluna
         delay_ms(tempo);           //temporiza
         output_high(COL4);         //desliga coluna
         break;
      case 5:
         output_low(COL5);          //liga coluna
         delay_ms(tempo);           //temporiza
         output_high(COL5);         //desliga coluna
         break;
      case 6:
         output_low(COL6);          //liga coluna
         delay_ms(tempo);           //temporiza
         output_high(COL6);         //desliga coluna
         break;
   }
  	restart_WDT();					     //zera WDT
}

//******************************************************************************
// Funo para atualizar a matriz
// recebe 	- nada
// retorna 	- nada
//******************************************************************************
void atualiza_matriz(void){

   restart_WDT();	                     //zera WDT
   LINHAS &= 0xF0;

   if (horario.segundos >= 10){        // maior que dez
     LINHAS = horario.segundos % 10;   //atualiza unidade
     pisca(6);
     LINHAS = horario.segundos / 10;   //atualiza dezena
     pisca(5);
   }
   else{
     LINHAS = horario.segundos;        //atualiza unidade
     pisca(6);
     LINHAS = 0;
     pisca(5);
   }

   if (horario.minutos >= 10){
     LINHAS = horario.minutos % 10;
     pisca(4);
     LINHAS = horario.minutos / 10;
     pisca(3);
   }
   else{
     LINHAS = horario.minutos;
     pisca(4);
     LINHAS = 0;
     pisca(3);
   }

   if (horario.horas >= 10){
      LINHAS = horario.horas % 10;
      pisca(2);
      LINHAS = horario.horas / 10;
      pisca(1);
   }
   else{
      LINHAS = horario.horas;
      pisca(2);
      LINHAS = 0;
      pisca(1);
   }         
      
   restart_WDT();	                 //zera WDT
}

//******************************************************************************
// Funo principal
//******************************************************************************
void main(void) {

   set_tris_A(0b00110000);		     //configura pinos do port A como sada,
                                   //exceto RA4
   set_tris_B(0b00000011);		     //configura pinos do port B como sada,
                                   //exceto RB0, RB1

   setup_comparator(nc_nc_nc_nc);  //desabilita comparadores internos

	setup_wdt(WDT_2304MS);			  //habilita watch dog timer a cada 2 segundos
	restart_WDT();						  //zera WDT

	setup_timer_1(T1_DISABLED);     //timer 1 desabilitado


	setup_timer_2(T2_DISABLED,0,1); //timer 2 desabilitado
	restart_WDT();						  //zera WDT

   //habilita interrupes
   enable_interrupts(INT_EXT);     //habilita interrupo no port RB0
   enable_interrupts(GLOBAL);      //habilita interrupo global
   restart_WDT();						  //zera WDT

   horario.horas    = 0;           //inicia horario
   horario.minutos  = 0;
   horario.segundos = 0;

   conta=0;                         //zera contador de interrupes
   est_ajt=0;                       //inicia mquina de estado
   prima=0;                         //no configurou nada ainda

   zera_lin_col();                  //limpa pinos de I/O de controle da
                                    //matriz de LEDS

   delay_ms(1000);                  //aguarda para ligar

   restart_WDT();	                  //zera WDT

   if (input(F24H)){                //sem jumper?
      hora_final = HF12;            //formato 12 horas
      hora_inicial= HI12;
   }
   else{                             //com jumper?
      hora_final = HF24;             //formato 24 horas
      hora_inicial = HI24;
   }

//******************************************************************************
// Inicio do lao principal
// A leitura do DS1620 e o envio do dado  feita via interrupo do timer 0
// O piscar do LED  controlado via interrupo do timer 1
// A recepo e retorno do caracter de eco  feita via int RDA (rec RS-232)
//******************************************************************************

   while(true){                     //faz eternamente

      //entra no modo ajuste
      if ((!input(AJT1)) || (!input(AJT2)))
         ajuste_horario();         //ajusta horrio

      if (prima)                    //se j acertou hora
         atualiza_matriz();         //atualiza matriz de LEDs
      else                          //seno pisca para informar necessidade
         pisca_ajuste();            //do ajuste

      restart_WDT();	               //zera WDT
   }
}
